"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const crypto_1 = __importDefault(require("crypto"));
const axios_1 = __importDefault(require("axios"));
exports.generateSignature = (path, secret, timestamp, method, hash) => {
    const joined = [timestamp, path, method, hash].join('');
    return crypto_1.default
        .createHmac('sha512', secret)
        .update(joined)
        .digest('hex');
};
exports.generateContentHash = (content = '') => {
    return crypto_1.default
        .createHash('sha512')
        .update(JSON.stringify(content))
        .digest('hex');
};
const prefix = 'https://api.bittrex.com/v3';
var HttpMethod;
(function (HttpMethod) {
    HttpMethod["GET"] = "GET";
    HttpMethod["POST"] = "POST";
    HttpMethod["PUT"] = "PUT";
    HttpMethod["DELETE"] = "DELETE";
    HttpMethod["PATCH"] = "PATCH";
    HttpMethod["HEAD"] = "HEAD";
    HttpMethod["TRACE"] = "TRACE";
    HttpMethod["OPTIONS"] = "OPTIONS";
    HttpMethod["CONNECT"] = "CONNECT";
})(HttpMethod = exports.HttpMethod || (exports.HttpMethod = {}));
exports.publicRequest = (path, params = {}) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const response = yield axios_1.default.get(`${prefix}/${path}`, { params });
        return response.data;
    }
    catch (e) {
        console.log(e);
        const error = new Error('Failed to execute request on exchange.');
        error.original = e;
        throw error;
    }
});
exports.privateRequest = (path, auth, params = {}, method = 'POST') => __awaiter(void 0, void 0, void 0, function* () {
    const { secret, key } = auth;
    const url = `${prefix}/${path}`;
    const timestamp = new Date().getTime();
    const hash = exports.generateContentHash(params);
    const signature = exports.generateSignature(url, secret, timestamp, method, hash);
    try {
        const response = yield axios_1.default.request({
            url,
            data: params,
            method,
            headers: {
                'Api-Key': key,
                'Api-Timestamp': timestamp,
                'Api-Content-Hash': hash,
                'Api-Signature': signature,
            },
        });
        return response.data;
    }
    catch (e) {
        console.log(e);
        const error = new Error('Failed to execute request on exchange.');
        error.original = e;
        throw error;
    }
});
